@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009-2011 Kuan-Chung Chiu <buganini@gmail.com>
+ * Copyright (c) 2009-2013 Kuan-Chung Chiu <buganini@gmail.com>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -231,6 +231,7 @@ py_bsdconv_conv(PyObject *self, PyObject *args)
 	ins->input.data=s;
 	ins->input.len=l;
 	ins->input.flags=0;
+	ins->input.next=NULL;
 	ins->flush=1;
 	bsdconv(ins);
 
@@ -254,6 +255,7 @@ py_bsdconv_conv_chunk(PyObject *self, PyObject *args)
 	ins->input.data=s;
 	ins->input.len=l;
 	ins->input.flags=0;
+	ins->input.next=NULL;
 	bsdconv(ins);
 
 	r=Py_BuildValue("s#",ins->output.data, ins->output.len);
@@ -276,6 +278,7 @@ py_bsdconv_conv_chunk_last(PyObject *self, PyObject *args)
 	ins->input.data=s;
 	ins->input.len=l;
 	ins->input.flags=0;
+	ins->input.next=NULL;
 	ins->flush=1;
 	bsdconv(ins);
 
@@ -330,6 +333,7 @@ py_bsdconv_conv_file(PyObject *self, PyObject *args)
 		ins->input.data=in;
 		ins->input.len=fread(in, 1, IBUFLEN, inf);
 		ins->input.flags|=F_FREE;
+		ins->input.next=NULL;
 		if(ins->input.len==0){
 			ins->flush=1;
 		}
@@ -363,6 +367,7 @@ py_bsdconv_testconv(PyObject *self, PyObject *args)
 	ins->input.data=s;
 	ins->input.len=l;
 	ins->input.flags=0;
+	ins->input.next=NULL;
 	ins->flush=1;
 	bsdconv(ins);
 
@@ -384,6 +389,7 @@ py_bsdconv_testconv_chunk(PyObject *self, PyObject *args)
 	ins->input.data=s;
 	ins->input.len=l;
 	ins->input.flags=0;
+	ins->input.next=NULL;
 	bsdconv(ins);
 
 	Py_INCREF(Py_None);
@@ -404,6 +410,7 @@ py_bsdconv_testconv_chunk_last(PyObject *self, PyObject *args)
 	ins->input.data=s;
 	ins->input.len=l;
 	ins->input.flags=0;
+	ins->input.next=NULL;
 	ins->flush=1;
 	bsdconv(ins);
 
@@ -433,6 +440,7 @@ py_bsdconv_testconv_file(PyObject *self, PyObject *args)
 		ins->input.data=in;
 		ins->input.len=fread(in, 1, IBUFLEN, inf);
 		ins->input.flags|=F_FREE;
+		ins->input.next=NULL;
 		if(ins->input.len==0){
 			ins->flush=1;
 		}
@@ -447,15 +455,46 @@ py_bsdconv_testconv_file(PyObject *self, PyObject *args)
 }
 
 static PyObject *
-py_bsdconv_info(PyObject *self, PyObject *args)
+py_bsdconv_counter(PyObject *self, PyObject *args)
 {
 	static PyObject *r;
+	char *k=NULL;
 	struct bsdconv_instance *ins;
 	ins=((Bsdconv *) self)->ins;
-	r=Py_BuildValue("{s:i,s:i,s:d,s:i,s:i,s:i}","ierr",ins->ierr,"oerr",ins->oerr,"score",ins->score,"full",ins->full,"half",ins->half,"ambi",ins->ambi);
+
+	if (!PyArg_ParseTuple(args, "|s", &k))
+		return NULL;
+
+	if (k!=NULL){
+		bsdconv_counter_t *v=bsdconv_counter(ins, k);
+		r=PyLong_FromSize_t(*v);
+	}else{
+		r=PyDict_New();
+		struct bsdconv_counter_entry *p=ins->counter;
+		while(p){
+			PyDict_SetItem(r, PyUnicode_FromString(p->key), PyLong_FromSize_t(p->val));
+			p=p->next;
+		}
+	}
 	return r;
 }
 
+static PyObject *
+py_bsdconv_counter_reset(PyObject *self, PyObject *args)
+{
+	char *k=NULL;
+	struct bsdconv_instance *ins;
+	ins=((Bsdconv *) self)->ins;
+
+	if (!PyArg_ParseTuple(args, "|s", &k))
+		return NULL;
+
+	bsdconv_counter_reset(ins, k);
+
+	Py_INCREF(Py_None);
+	return Py_None;
+}
+
 static PyObject *
 py_bsdconv_error(PyObject *self, PyObject *args)
 {
@@ -472,7 +511,7 @@ py_bsdconv_valid(PyObject *self, PyObject *args)
 {
 	struct bsdconv_instance *ins;
 	ins=((Bsdconv *) self)->ins;
-	
+
 	if(ins!=NULL)
 		return 1;
 	else
@@ -500,8 +539,10 @@ static PyMethodDef Bsdconv_methods[] = {
 		PyDoc_STR("testconv_chunk_last(s) -> Perform test conversion without initializing")},
 	{"testconv_file",	py_bsdconv_testconv_file,	METH_VARARGS,
 		PyDoc_STR("testconv_file(from_file) -> Perform test conversion with given filename")},
-	{"info",	py_bsdconv_info,	METH_VARARGS,
-		PyDoc_STR("info(cd) -> Return conversion info")},
+	{"counter",	py_bsdconv_counter,	METH_VARARGS,
+		PyDoc_STR("counter([name]) -> Return conversion info")},
+	{"counter_reset",	py_bsdconv_counter_reset,	METH_VARARGS,
+		PyDoc_STR("counter_reset([name]) -> Reset counter, if no name supplied, all counters will be reset")},
 	{NULL,		NULL}		/* sentinel */
 };
 
